import os
import socket
import cv2
import argparse
import sys
import threading
import queue
import datetime
import time
import shutil
import numpy as np
from collections import Counter
from yolov5_tflite_inference import yolov5_tflite
import configparser


# 平山 設定ファイル読み込み 2023.4.18
config = configparser.ConfigParser()
config.read("/taksys/PEdetection/config_Armadillo.ini")
config_fk_path = config["mosquitto"]["fk"]

# 機器番号の取得
with open(config_fk_path, "r")as fk:
    fk_no = fk.readline().rstrip()

WINDOW_NAME = 'endo_detect'
MODEL_NAME = 'Ibi+GB009-int8.tflite'
DIR_NAME = os.path.dirname(__file__)

# layout settings for armadillo-demo-experience
CAMERA_WIDTH = 1280
CAMERA_HEIGHT = 720

# フレームサイズ
FRAME_SIZE = 1
# 発報判断秒数
ALARM_SEC = 5
CLEAR_SEC = 10
IGNORE_SEC = 15

# パトライトIP
PAT_IP_ADRESS = "192.168.63.248"
PAT_PORT_NO = 10000

#STATE定義
STATE_NONE = -1
STATE_WORK = 0
STATE_RISK = 1

#クラス名定義
CLS_Goggle_Out = "Goggle_Out"
CLS_Goggle_Set = "Goggle_Set"
CLS_Glove_Out = "Glove_Out"
CLS_Glove_Set = "Glove_Set"
CLS_P_Drum = "P_Drum"

#発報パラメータ
config_time = 5
config_rate = 60
config_area = 150

#ソケット通信クラス定義
class SockClient:
    def __init__(self, timeout:int=10, buffer:int=1024):
        self.__socket = None
        self.__address = None
        self.__timeout = timeout
        self.__buffer = buffer

    def connect(self, address, family:int, typ:int, proto:int):
        self.__address = address
        self.__socket = socket.socket(family, typ, proto)
        self.__socket.settimeout(self.__timeout)
        self.__socket.connect(self.__address)

    def send(self, message:str="") -> None:
        flag = False
        while True:
            if message == "":
                message_send = input("> ")
            else:
                message_send=message
                flag = True
            self.__socket.send(message_send.encode('utf-8'))
            message_recv = self.__socket.recv(self.__buffer).decode('utf-8')
            self.received(message_recv)
            if(message_recv == chr(0x6)):
                print("正常")
            else:
                print("異常通信")
            if flag:
                break
        try:
            self.__socket.shutdown(socket.SHUT_RDWR)
            self.__socket.close()
        except:
            pass

    def received(self, message:str):
        print(message)

class InetClient(SockClient):
    def __init__(self, host:str="0.0.0.0", port:int=8080) -> None:
        self.server=(host,port)
        super().__init__(timeout=10, buffer=1024)
        super().connect(self.server, socket.AF_INET, socket.SOCK_STREAM, 0)

def CreateSendText(mode):
    if mode == STATE_NONE:
        sendText = "XXS" + chr(0x0) + chr(0x0) + chr(0x6) + chr(0x0) + chr(0x0) + chr(0x0) + chr(0x0) + chr(0x0) + chr(0x0)
    elif mode == STATE_WORK:
        sendText = "XXS" + chr(0x0) + chr(0x0) + chr(0x6) + chr(0x0) + chr(0x0) + chr(0x1) + chr(0x0) + chr(0x0) + chr(0x0)
    elif mode == STATE_RISK:
        sendText = "XXW" + chr(0x0) + chr(0x0) + chr(0xA) + chr(0x1) + chr(0x0) + chr(0x0) + chr(0x0) + chr(0x0) + chr(0x0) + chr(0x1) + chr(0x0) + chr(0x0) + chr(0x1)
    elif mode == -2:
        sendText = "XXS" + chr(0x0) + chr(0x0) + chr(0x6) + chr(0x0) + chr(0x1) + chr(0x0) + chr(0x0) + chr(0x0) + chr(0x0)

    return sendText

def SendCommand(mode):        # 点灯コマンド
    try:
        cli = InetClient(host=PAT_IP_ADRESS, port=PAT_PORT_NO)
        sendText = CreateSendText(mode)
        cli.send(sendText)
    except Exception as e:
        setLog(e)

#boxの取得
def GetNearestBoxIndex(point, class_names, boxes):
    goggle_nearest_dist = None
    google_nearset_class_name = None
    glove_nearest_dist = None
    glove_nearest_class_name = None

    result = False
    
    for class_name, box in zip(class_names, boxes):
        # 矩形の中心座標を計算する
        center = np.array([(box[0] + box[2]) / 2, (box[1] + box[3]) / 2])        
        # 点と中心座標の距離を計算する
        dist = np.linalg.norm(center - point)
        
        if class_name == CLS_Goggle_Out or class_name == CLS_Goggle_Set:
            # 距離が一番近い矩形を更新する
            if goggle_nearest_dist is None or dist < goggle_nearest_dist:
                goggle_nearest_dist = dist
                google_nearset_class_name = class_name
        elif class_name == CLS_Glove_Out or class_name == CLS_Glove_Set:
            # 距離が一番近い矩形を更新する
            if glove_nearest_dist is None or dist < glove_nearest_dist:
                glove_nearest_dist = dist
                glove_nearest_class_name = class_name

    #両方とも見つかればセーフ
    if google_nearset_class_name == CLS_Goggle_Set and glove_nearest_class_name == CLS_Glove_Set:
        result = True
    elif google_nearset_class_name == CLS_Goggle_Set and glove_nearest_dist == None:
        #片方しか見つかっていない場合もセーフ
        result = True
    elif google_nearset_class_name == CLS_Glove_Set and goggle_nearest_dist == None:
        result = True
        
    return result

#現在状況の文字列取得
def getAlartStateText(state):
        if state == STATE_NONE:
            return "None"
        elif state == STATE_WORK:
            return "Safe"
        elif state == STATE_RISK:
            return "Danger!"
        return "None"

#ログ出力
def setLog(state):
        new_dir_path_log = '/dev/sd/takcam/log'            #logファイル格納先パス
        os.makedirs(new_dir_path_log, exist_ok=True)       #logディレクトリ作成
        now = datetime.datetime.now().replace(microsecond=0)
        f = open(new_dir_path_log + '/log.txt', 'a')
        f.write(str(now) + '    :   ' + str(state) + '\n')
        f.close()

# 平山 mqttLog 作成 2023.4.18
# Mqtt へ ログ出力
def mqttLog(state, thumb_name, mov_name):
        
        new_dir_path_mqttlog = '/dev/sd/takcam/mqttlog'                             #logファイル格納先パス
        os.makedirs(new_dir_path_mqttlog, exist_ok=True)                            #logディレクトリ作成
        now = datetime.datetime.now().replace(microsecond=0)
        log_timestamp = now.strftime('%Y%m%d%H%M%S')
        file_log = os.path.join(new_dir_path_mqttlog, log_timestamp + '.txt')       #logファイルの作成

        if state == STATE_RISK or state == STATE_WORK:
            f = open(file_log, "w")
            f.write(str(fk_no) + ',' + str(now) + ',' + thumb_name + ',' + mov_name + ','+ str(state))
        
        f.close()

#サムネイル切り出し
def makeThumbnail(videoPath, outputPath):
    cap = cv2.VideoCapture(videoPath)
    if not cap.isOpened(): return

    # 最初のフレームを読み込む
    _, img = cap.read()
    # imgのshapeは (高さ, 横幅, 3)

    # 画像ファイルで書き出す
    cv2.imwrite(outputPath, img)


class FrameHandler:
    def __init__(
            self,
            len_thres,
            imgsrc,
            visualize_score=False,
            visualize_length=False):
        self.picture_flag = False
        if type(imgsrc) is str:
            self.current_image = cv2.imread(imgsrc)
            self.camera = None
        else:
            self.camera = imgsrc
            self.current_image = None
        self.len_thres = len_thres
        self.visualize_score = visualize_score
        self.visualize_length = visualize_length
        self.infer_result = None

    def set_infer_result(self, data):
        self.infer_result = data

    def _calc_distance(self, box):
        return np.linalg.norm(box[2:] - box[:2])

    def run(self, cw, ch):
        #ST 20230509 kmiyazato ドラム缶の検出ロジックの変更
        clearsec = 0
        tempdrum = None
        tempdrumtime = datetime.datetime.now()
        #ED 20230509 kmiyazato ドラム缶の検出ロジックの変更
        chkframes = [] #フレーム単位の判定結果
        alertbuffer = [] #発報単位の判定結果
        tempState = STATE_NONE #１フレーム前の状態　
        nowAlartState = STATE_NONE #現在の発報状態
        safeRecordFlg = False
        prmSafeRecordFlg = False
        alertRecordFlg = False
        prmAlertRecordFlg = False
        startflg = False
        firstTimeStart = True
        jpgflg = False
        thumbflg = False
        restflg = False
        alarmflg = False
        safeToAlert = False
        prmDrumCenter = np.array([0,0])

        firtStartTime = time.time() #起動時刻

        if not self.camera is None:
            #カメラ仕様
            fps = int(self.camera.get(cv2.CAP_PROP_FPS))            #カメラのFPSを取得
            w = int(self.camera.get(cv2.CAP_PROP_FRAME_WIDTH))      #カメラの横幅を取得
            h = int(self.camera.get(cv2.CAP_PROP_FRAME_HEIGHT))     #カメラの縦幅を取得
            #fourcc = cv2.VideoWriter_fourcc('m', 'p', '4', 'v')     #動画の仕様
            
        #ディレクトリ作成
        videoPath = '/dev/sd/takcam/video/'
        new_dir_path_safe = videoPath + 'safe/mov/'             #安全行動発報時録画ディレクトリパス
        new_dir_path_alert = videoPath + 'alert/mov/'           #不安全行動発報時録画ディレクトリパス
        new_dir_path_logs = videoPath + 'logs/'                 #常時録画ディレクトリパス
        new_dir_path_safe_thumb = videoPath + 'safe/thumb/'     #安全行動発報時録画サムネディレクトリパス
        new_dir_path_alert_thumb = videoPath + 'alert/thumb/'   #不安全行動発報時録画サムネディレクトリパス
        os.makedirs(new_dir_path_safe, exist_ok=True)           
        os.makedirs(new_dir_path_safe_thumb, exist_ok=True)     
        os.makedirs(new_dir_path_alert, exist_ok=True)          
        os.makedirs(new_dir_path_alert_thumb, exist_ok=True)    
        os.makedirs(new_dir_path_logs, exist_ok=True)

        # 検出範囲の矩形
        xmin,xmax = 0,CAMERA_WIDTH 
        ymin,ymax = 0,CAMERA_HEIGHT 

        # iniファイル読み込み
        try:
            # ConfigParserのインスタンスを取得
            #config = configparser.ConfigParser()

            # 用意したconfig.iniを読み込み
            #config.read("./config_Armadillo.ini")

            #変数configから、"ALERT"セクションの"time", "rate", "area"を取り出す
            config_time = int(config["ALERT"]["time"])
            config_rate = int(config["ALERT"]["rate"])
            config_area = int(config["ALERT"]["area"])
        except:
            pass           
        
        #初回起動時はパトライトを安全行動で点灯
        SendCommand(STATE_WORK) 

        try:
            while True:
                if firstTimeStart == True:
                    if time.time() - firtStartTime > 10:
                        #起動から10秒後パトライト消灯
                        SendCommand(STATE_NONE)
                        firstTimeStart = False

                if not self.camera is None:
                    ret, self.current_image = self.camera.read()

                    #if jpgflg == False:
                    #    jpg_start = cv2.getTickCount()
                    #    jpgflg = True
                    
                    #常時録画の設定
                    if startflg == False:
                        #動画ファイルの保存設定
                        recording_time = 1200                                  #録画時間(seconds) 
                        now = datetime.datetime.now()
                        movname = new_dir_path_logs + now.strftime('%Y%m%d%H%M%S')  + '.mp4'       #動画ファイル保存先
                        pipeline = f"appsrc ! autovideoconvert ! video/x-raw,width={w},height={h},framerate={fps}/1 ! queue ! vpuenc_h264 ! h264parse ! qtmux ! filesink location={movname} sync=false" 
                        video = cv2.VideoWriter(pipeline, 0, fps, (w, h))       #動画の仕様
                        recordinglogs_start = cv2.getTickCount()                #録画開始時刻 
                        startflg = True

                    #常時録画開始
                    if startflg == True:
                        #動画保存(バウンディングボックスなしで保存)
                        video.write(self.current_image)
                        #JPEG保存
                        #jpeg_name = new_dir_path_logs + datetime.datetime.now().strftime('%Y%m%d%H%M%S%f') + '.jpg'
                        #cv2.imwrite(jpeg_name, frame)
                        elapsed_time = (cv2.getTickCount() - recordinglogs_start) / cv2.getTickFrequency()
                        if elapsed_time >= recording_time:
                            #JPEG書き出し
                            #cv2.imwrite(jpeg_name, self.current_image)
                            #録画時間を超えると解放
                            startflg = False
                            video.release()

                    # 平山 alertTime_stamp 作成 2023.4.18
                    # 平山 サムネイル・動画名の頭に機械番号追加 20230427
                    #不安全行動発報時の録画設定
                    if alertRecordFlg != prmAlertRecordFlg:
                        #動画ファイルの保存設定
                        time_limit = 30                                         #録画時間(seconds) 
                        alertTime = datetime.datetime.now()
                        alertTime_stamp = alertTime.strftime('%Y%m%d%H%M%S')
                        alertmovname_path =  new_dir_path_alert + fk_no + alertTime_stamp +'.mp4'       #動画ファイル保存先
                        alertthumbname_path = new_dir_path_alert_thumb + fk_no + alertTime_stamp + '.jpg'
                        alertpipeline = f"appsrc ! autovideoconvert ! video/x-raw,width={w},height={h},framerate={fps}/1 ! queue ! vpuenc_h264 ! h264parse ! qtmux ! filesink location={alertmovname_path} sync=false" 
                        video_alert = cv2.VideoWriter(alertpipeline, 0, fps, (w, h))       #動画の仕様
                        recordingalert_start = cv2.getTickCount()               #録画開始時刻 
                        prmAlertRecordFlg = alertRecordFlg
                        #alertdir = os.path.join(new_dir_path_alert , alertTime.strftime('%Y%m%d%H%M%S'))
                        #os.makedirs(alertdir, exist_ok=True)

                    #不安全行動発報時の録画開始
                    if alertRecordFlg == True:
                        #動画保存(バウンディングボックスありで保存)
                        video_alert.write(frame)
                        '''jpeg_name = datetime.datetime.now().strftime('%Y%m%d%H%M%S%f') + '.jpg'
                        jpeg_name_alert = os.path.join(alertdir, jpeg_name)
                        jpg_time = (cv2.getTickCount() - jpg_start) / cv2.getTickFrequency()
                        if jpg_time >= 0.5:
                            cv2.imwrite(jpeg_name_alert, frame)
                            if thumbflg == False:
                                #初回はサムネをコピー
                                thumbflg = True
                                shutil.copy(jpeg_name_alert, os.path.join(new_dir_path_alert_thumb, jpeg_name))
                            jpgflg = False'''
                        elapsed_time = (cv2.getTickCount() - recordingalert_start) / cv2.getTickFrequency()
                        if elapsed_time >= time_limit:
                            #録画時間を超えると解放
                            alertRecordFlg = False
                            prmAlertRecordFlg = False
                            video_alert.release()

                            # 平山 mqttLog() 追加 2023.4.18
                            #サムネイル格納
                            makeThumbnail(alertmovname_path, alertthumbname_path)
                            mqttLog(STATE_RISK, fk_no + alertTime_stamp + '.jpg',fk_no + alertTime_stamp + '.mp4')

                    # 平山 safeTime_stamp 作成 2023.4.18
                    # 平山 サムネイル・動画名の頭に機械番号追加 20230427 
                    #安全行動発報時の録画設定
                    if safeRecordFlg != prmSafeRecordFlg:
                        #動画ファイルの保存設定
                        time_limit = 30                                         #録画時間(seconds) 
                        safeTime = datetime.datetime.now()
                        safeTime_stamp = safeTime.strftime('%Y%m%d%H%M%S')
                        safemovname_path = new_dir_path_safe + fk_no + safeTime_stamp + '.mp4'         #動画ファイル保存先
                        safethumbname_path = new_dir_path_safe_thumb + fk_no + safeTime_stamp + '.jpg'
                        safepipeline = f"appsrc ! autovideoconvert ! video/x-raw,width={w},height={h},framerate={fps}/1 ! queue ! vpuenc_h264 ! h264parse ! qtmux ! filesink location={safemovname_path} sync=false" 
                        video_safe = cv2.VideoWriter(safepipeline, 0, fps, (w, h))       #動画の仕様
                        recordingsafe_start = cv2.getTickCount()              #録画開始時刻 
                        prmSafeRecordFlg = safeRecordFlg
                        #safedir = os.path.join(new_dir_path_safe, safeTime.strftime('%Y%m%d%H%M%S'))
                        #os.makedirs(safedir, exist_ok=True)

                    #安全行動発報時の録画開始
                    if safeRecordFlg == True:
                        #動画保存(バウンディングボックスありで保存)
                        video_safe.write(frame)
                        '''jpeg_name = datetime.datetime.now().strftime('%Y%m%d%H%M%S%f') + '.jpg'
                        jpeg_name_safe = os.path.join(safedir, jpeg_name)
                        jpg_time = (cv2.getTickCount() - jpg_start) / cv2.getTickFrequency()
                        if jpg_time >= 0.5:
                            cv2.imwrite(jpeg_name_safe, frame)
                            if thumbflg == False:
                                #初回はサムネをコピー
                                thumbflg = True
                                shutil.copy(jpeg_name_safe, os.path.join(new_dir_path_safe_thumb, jpeg_name))
                            jpgflg = False'''
                        elapsed_time = (cv2.getTickCount() - recordingsafe_start) / cv2.getTickFrequency()
                        if elapsed_time >= time_limit:
                            #録画時間を超えると解放
                            safeRecordFlg = False
                            prmSafeRecordFlg = False
                            video_safe.release()

                            # 平山 mqttLog() 追加 2023.4.18
                            #サムネイル格納
                            makeThumbnail(safemovname_path, safethumbname_path)
                            mqttLog(STATE_WORK, fk_no + safeTime_stamp + '.jpg', fk_no + safeTime_stamp + '.mp4')
                            #thumbflg = False
                    
                    if not ret:
                        print('video end.')
                        sys.exit(0)
                frame = cv2.resize(self.current_image.copy(), (cw, ch))

                if self.infer_result is None:
                    continue
                boxes, scores, class_names, pred_time = self.infer_result

                ##############検知発報系##############
                chkframe = STATE_NONE #-1:何も見つかっていない状態 0:保護具着用 1:保護具未着用

                #状態判断
                flag_GoggleOut = False
                flag_GoggleSet = False
                flag_GloveOut = False
                flag_GloveSet = False
                flag_PDrum = False
                
                if CLS_Goggle_Out in class_names:
                    flag_GoggleOut = True
                if CLS_Goggle_Set in class_names:
                    flag_GoggleSet = True
                if CLS_Glove_Out in class_names:
                    flag_GloveOut = True
                if CLS_Glove_Set in class_names:
                    flag_GloveSet = True
                if CLS_P_Drum in class_names:
                    flag_PDrum = True

                #ST 20230509 kmiyazato ドラム缶の検出ロジックの変更
                #一度ドラム缶を検出した場合はドラム缶があると判定する
                if flag_PDrum == False:
                    chkframe = STATE_NONE
                    clearsec = tempdrumtime + datetime.timedelta(seconds=CLEAR_SEC)
                    if datetime.datetime.now() >= clearsec:
                        #未検知が指定の数字以上続いた場合は、バッファを消す
                        alertbuffer.clear()
                        tempdrum = None
                        tempdrumtime = datetime.datetime.now()
                        if nowAlartState != STATE_NONE:
                            #消灯する
                            SendCommand(STATE_NONE)
                            nowAlartState = STATE_NONE
                            setLog(STATE_NONE)
                else:
                    tempdrumtime = datetime.datetime.now()
                    for class_name, box, score in zip(class_names, boxes, scores):
                        if class_name == CLS_P_Drum:
                            tempdrum = (class_name, box, score)
                            break
                #ED 20230509 kmiyazato ドラム缶の検出ロジックの変更

                #ドラムがある場合
                if flag_PDrum == True or tempdrum is not None:
                    #保護具未着用者が見つからなければ安全行動
                    chkframe = STATE_WORK

                    #ST 20230509 kmiyazato ドラム缶の検出ロジックの変更
                    tbox = None
                    if flag_PDrum == True:
                        for class_name, box in zip(class_names, boxes):
                            if class_name == CLS_P_Drum: 
                                tbox = box
                                break
                    elif tempdrum is not None:
                        tbox = tempdrum[1]
                    # ドラムの中心座標を取得
                    drumCenter = np.array([(tbox[0] + tbox[2]) / 2, (tbox[1] + tbox[3]) / 2])
                    drumWeight = tbox[2] - tbox[0]
                    #   ドラム缶から検出範囲を計算
                    deltax =  drumWeight/2 * 100/100
                    xmin = int(tbox[0] - deltax)
                    xmax = int(tbox[2] + deltax)
                    #ED 20230509 kmiyazato ドラム缶の検出ロジックの変更
                    
                    #ST 20230605 ドラム缶領域内の検出判定ロジックの変更
                    InArea = False
                    for class_name, box in zip(class_names, boxes):
                        # ゴーグルが検出範囲内に入っているか取得
                        if class_name == CLS_Goggle_Out:
                            if (box[0] >= xmin and box[0] <= xmax) or (box[0] <= xmin and box[2] >= xmin) or (box[0] <= xmax and box[2] >= xmax):
                                InArea = True
                                break
                        # 手袋が検出範囲内に入っているか取得
                        if class_name == CLS_Glove_Out:
                            if (box[0] >= xmin and box[0] <= xmax) or (box[0] <= xmin and box[2] >= xmin) or (box[0] <= xmax and box[2] >= xmax):
                                InArea = True
                                break
                     #ED 20230605 ドラム缶領域内の検出判定ロジックの変更

                    drumMVflg = False
                    # if (drumCenter[0] <= prmDrumCenter[0] - 10) or (drumCenter[0] >= prmDrumCenter[0] + 10) or (drumCenter[1] <= prmDrumCenter[1] - 10) or (drumCenter[1] >= prmDrumCenter[1] + 10):
                    #     # ドラムの中心が10px以上離れていると移動中と認識
                    #     drumMVflg = True
                    #     prmDrumCenter = drumCenter

                    # デバッグ用に検出範囲に赤枠
                    cv2.rectangle(frame,(xmin,ymin),(xmax,ymax),(0,0,255),2) 

                    if drumMVflg == False:
                        #ドラムは移動中でない
                        if InArea == True:
                            # 物体検知した画素があれば、範囲内の検出処理を実施    
                            if flag_GoggleSet == False and flag_GloveSet == False:
                                #着用者が一人もいない場合
                                if flag_GoggleOut == True or flag_GloveOut == True:
                                    #未着用者が見つかれば警告状態
                                    chkframe = STATE_RISK
                            elif flag_GoggleSet == True and flag_GloveSet == True:
                                #着用者がいた場合
                                if flag_GoggleOut == True or flag_GloveOut == True:
                                    #未着用者がいた場合は、ドラム缶からの距離と近いほうを採用
                                    #idx = GetNearestBoxIndex(drumCenter,class_names,boxes)
                                    #if GetNearestBoxIndex(drumCenter,class_names,boxes) == True:
                                    #    chkframe = STATE_WORK
                                    #else:
                                        chkframe = STATE_RISK
                                else:
                                    chkframe = STATE_WORK
                            else:
                                #保護具の片方しか見つからない場合
                                if flag_GoggleOut == True or flag_GloveOut == True:
                                    #未着用者がいた場合は、中心からの距離と近いほうを採用
                                    #idx = GetNearestBoxIndex(drumCenter,class_names,boxes)
                                    #if GetNearestBoxIndex(drumCenter,class_names,boxes) == True:
                                    #    chkframe = STATE_WORK
                                    #else:
                                        chkframe = STATE_RISK
                                else:
                                    chkframe = STATE_WORK
                #ドラムがない場合は未検出
                else:
                    chkframe = STATE_NONE
                    
                #発報判断
                chkframes.append(chkframe)
                if len(chkframes) == FRAME_SIZE:
                    #フレームサイズに到達したら判定
                    mcv = Counter(chkframes).most_common(1)[0][0]
                    
                    # #未検知が続く場合のロジック
                    # if tempState == STATE_NONE and mcv == STATE_NONE:
                    #     clearSec += 1
                    # else:
                    #     clearSec = 0
                    # if clearSec >= CLEAR_SEC:
                    #     #未検知が指定の数字以上続いた場合は、バッファを消す
                    #     alertbuffer.clear()
                    #     clearSec = 0
                    #     if nowAlartState != STATE_NONE:
                    #         #消灯する
                    #         SendCommand(STATE_NONE)
                    #         nowAlartState = STATE_NONE
                    #         setLog(STATE_NONE)
                        
                    #バッファに入れる(バッファには非検知を入れない)
                    if mcv != STATE_NONE:
                        alertbuffer.append(mcv)

                    #バッファがアラーム秒を超えたら通知
                    if alertbuffer.count(mcv) >= 1:
                        if alarmflg == False:
                            alarm_start = cv2.getTickCount()
                            alarmflg = True
                        trigger_time = (cv2.getTickCount() - alarm_start) / cv2.getTickFrequency()
                        if trigger_time >= config_time:
                            if mcv != STATE_NONE:
                                # 発報判断を指定時間無視する
                                if nowAlartState == STATE_RISK: 
                                    safeToAlert = True
                                    if trigger_time >= IGNORE_SEC:
                                        safeToAlert = False
                                else:
                                    safeToAlert = False
                                if safeToAlert == False:
                                    # 赤発報後は指定時間発報判断を無視する
                                    if  alertbuffer.count(STATE_RISK) >= len(alertbuffer) * config_rate/100:
                                        nowState = STATE_RISK
                                    else:
                                        nowState = STATE_WORK

                                    if nowAlartState != nowState:
                                        SendCommand(nowState)
                                        alarmflg = False
                                        nowAlartState = nowState
                                        alertbuffer.clear()
                                        #最後だけ残す
                                        alertbuffer.append(nowState)
                                        if nowAlartState == STATE_RISK:
                                            alertRecordFlg = True
                                        else:
                                            safeRecordFlg = True
                                        setLog(nowAlartState)
                                    else:
                                        alertbuffer.pop(0)
                                else:
                                    alertbuffer.pop(0)
                    else:
                        alarmflg = False

                    tempState = mcv
                    chkframes.clear()


                ##############表示系##############
                setCount = 0
                outCount = 0
                #バウンディボックス表示
                for box, score, class_name in zip(boxes, scores, class_names):
                    box = box.astype(np.int32)
                    #ST 色条件変更
                    if class_name == CLS_Goggle_Set:
                        color = (0,255,0)
                        setCount += 1
                    elif class_name == CLS_Goggle_Out:
                        color = (255,0,255)
                        outCount += 1
                    elif class_name == CLS_Glove_Set:
                        color = (0,255,0)
                        setCount += 1
                    elif class_name == CLS_Glove_Out:
                        color = (255,0,255)
                        outCount += 1
                    else:
                        color = (255,0,0)

                    #ED 色条件変更
                    cv2.rectangle(frame, box[:2], box[2:], color, 2)
                    
                    #バウンディボックス上に表示する文字列の作成
                    box_text = ''
                    if self.visualize_score:
                        box_text += '{}:{:.2f} '.format(class_name,score)
                    if box_text != '':
                        (tw, th), _ = cv2.getTextSize(box_text, cv2.FONT_HERSHEY_SIMPLEX, 0.5, 1)
                        cv2.rectangle(
                            frame,
                            box[:2],
                            (box[0] + (tw + 3), box[1] - (th + 2)),
                            color,
                            thickness=-1
                        )
                        cv2.putText(
                            frame,
                            text=box_text,
                            org=box[:2],
                            fontFace=cv2.FONT_HERSHEY_SIMPLEX,
                            fontScale=0.5,
                            color=(255, 255, 255),
                            thickness=1,
                            lineType=cv2.LINE_AA
                        )

                #ST 20230509 kmiyazato ドラム缶の検出ロジックの変更
                if flag_PDrum == False and tempdrum is not None:
                    color = (255,0,0)
                    box = tempdrum[1].astype(np.int32)
                    cv2.rectangle(frame, box[:2], box[2:], color, 2)
                    box_text = ''
                    if self.visualize_score:
                        box_text += '{}:{:.2f} '.format(tempdrum[0],tempdrum[2])
                    if box_text != '':
                        (tw, th), _ = cv2.getTextSize(box_text, cv2.FONT_HERSHEY_SIMPLEX, 0.5, 1)
                        cv2.rectangle(
                            frame,
                            box[:2],
                            (box[0] + (tw + 3), box[1] - (th + 2)),
                            color,
                            thickness=-1
                        )
                        cv2.putText(
                            frame,
                            text=box_text,
                            org=box[:2],
                            fontFace=cv2.FONT_HERSHEY_SIMPLEX,
                            fontScale=0.5,
                            color=(255, 255, 255),
                            thickness=1,
                            lineType=cv2.LINE_AA
                        )
                #ED 20230509 kmiyazato ドラム缶の検出ロジックの変更

                #fpsの表示
                frame_height, frame_width, _ = frame.shape
                font_scale = 1
                font_face = cv2.FONT_HERSHEY_DUPLEX
                pred_text = 'FPS: {:.2f}'.format(1 / pred_time)
                (tw_d, th_d), _ = cv2.getTextSize(pred_text, font_face, font_scale, 1)            
                cv2.rectangle(frame, (frame_width - tw_d, 0), (frame_width, th_d), (255, 255, 255), thickness=-1)
                cv2.putText(
                    frame,
                    text=pred_text,
                    org=(frame_width - tw_d, th_d),
                    fontFace=font_face,
                    fontScale=font_scale,
                    color=(0, 0, 255),
                    thickness=1,
                    lineType=cv2.LINE_AA
                )

                #状態の表示
                state_text = 'State: {}'.format(getAlartStateText(nowAlartState))
                (tw_d, th_d), _ = cv2.getTextSize(state_text, font_face, font_scale, 1)
                cv2.rectangle(frame, (frame_width - tw_d, frame_height - th_d - 4), (frame_width, frame_height), (255, 255, 255), thickness=-1)
                cv2.putText(
                    frame,
                    text=state_text,
                    org=(frame_width - tw_d, frame_height - 2),
                    fontFace=font_face,
                    fontScale=font_scale,
                    color=(0, 0, 255),
                    thickness=1,
                    lineType=cv2.LINE_AA
                )

                #時間の表示
                now_text = datetime.datetime.now().strftime('%Y/%m/%d %H:%M:%S')
                (tw_d, th_d), _ = cv2.getTextSize(state_text, font_face, font_scale, 1)
                cv2.rectangle(frame, (0, 0), (365, th_d), (255, 255, 255), thickness=-1)
                cv2.putText(
                    frame,
                    text=now_text,
                    org=(0, th_d),
                    fontFace=font_face,
                    fontScale=font_scale,
                    color=(0, 0, 255),
                    thickness=1,
                    lineType=cv2.LINE_AA
                )

                #デバッグ用の表示
                state_text = 'Debug: fs:{} / drum:{}|{}'.format(getAlartStateText(chkframe),flag_PDrum,tempdrumtime.strftime('%Y/%m/%d %H:%M:%S'))
                (tw_d, th_d), _ = cv2.getTextSize(state_text, font_face, font_scale, 1)
                cv2.putText(
                    frame,
                    text=state_text,
                    org=(0, frame_height - 2),
                    fontFace=font_face,
                    fontScale=font_scale,
                    color=(0, 0, 255),
                    thickness=1,
                    lineType=cv2.LINE_AA
                )
                
                cv2.imshow(WINDOW_NAME, frame)
                cv2.waitKey(1)
        except Exception as e:
            setLog(e)
            print(e)
            SendCommand(-2)


    def get_frame(self):
        return self.current_image


class ScrewDetector(yolov5_tflite): 
    def __init__(self, model_path, conf_thres, iou_thres, imgsrc):
        super().__init__(
            model_path,
            conf_thres=conf_thres,
            iou_thres=iou_thres
        )
        self.imgsrc = imgsrc
        self.q = queue.Queue()

    def run(self, cw, ch):
        while True:
            frame = self.imgsrc.get_frame()
            if frame is None:
                continue
            frame = cv2.resize(frame, (cw, ch))
            self.q.put(self.detect(frame))

    def get_infer_result(self):
        return self.q.get()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--model',
        type=str,
        default=MODEL_NAME
    )
    parser.add_argument(
        '--input_img',
        type=str
    )
    parser.add_argument(
        '--input_video',
        type=str
    )
    parser.add_argument(
        '--camera_id',
        type=int,
        default=2
    )
    parser.add_argument(
        '--width',
        type=int,
        default=CAMERA_WIDTH
    )
    parser.add_argument(
        '--height',
        type=int,
        default=CAMERA_HEIGHT
    )
    parser.add_argument(
        '--conf_thres',
        type=float,
        default=0.5
    )
    parser.add_argument(
        '--iou_thres',
        type=float,
        default=0.45
    )
    parser.add_argument(
        '--len_thres',
        type=int,
        default=130
    )
    parser.add_argument(
        '--visualize_score',
        action='store_true'
    )
    parser.add_argument(
        '--visualize_length',
        action='store_true'
    )
    args = parser.parse_args()
    args.visualize_score = True

    model_path = os.path.join(DIR_NAME, args.model)

    #WinST Windowsだとハングするので修正
    cv2.namedWindow(WINDOW_NAME, cv2.WINDOW_NORMAL)
    cv2.setWindowProperty(WINDOW_NAME, cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)
    cv2.moveWindow(WINDOW_NAME, 0, 0)
    #WinED Windowsだとハングするので修正

    if not args.input_img is None:
        src = args.input_img
    elif not args.input_video is None:
        src = cv2.VideoCapture(args.input_video)
    else:
        #WinST WindowsだとV4L2は認識しないので修正
        src = cv2.VideoCapture(args.camera_id, cv2.CAP_V4L2)
        #src = cv2.VideoCapture(args.camera_id)
        #WinST WindowsだとV4L2は認識しないので修正
        src.set(cv2.CAP_PROP_FRAME_WIDTH, CAMERA_WIDTH)
        src.set(cv2.CAP_PROP_FRAME_HEIGHT, CAMERA_HEIGHT)

    frame_handler = FrameHandler(
        args.len_thres,
        imgsrc=src,
        visualize_score=args.visualize_score,
        visualize_length=args.visualize_length
    )

    t_fhandler = threading.Thread(
        target=frame_handler.run,
        args=(args.width, args.height)
    )
    t_fhandler.start()

    inferencer = ScrewDetector(
        model_path=model_path,
        conf_thres=args.conf_thres,
        iou_thres=args.iou_thres,
        imgsrc=frame_handler
    )

    t_inferencer = threading.Thread(
        target=inferencer.run,
        args=(args.width, args.height)
    )
    t_inferencer.start()

    while True:
        frame_handler.set_infer_result(inferencer.get_infer_result())


if __name__ == '__main__':
    main()
